
#ifndef INTELLIVISION_H
#define INTELLIVISION_H

#include "AudioMixer.h"
#include "AY38900.h"
#include "AY38914.h"
#include "Cartridge.h"
#include "CP1610.h"
#include "ECS.h"
#include "EmulationDirector.h"
#include "HandController.h"
#include "Intellivoice.h"
#include "MemoryBus.h"
#include "Peripheral.h"
#include "PlugIn.h"
#include "ProcessorBus.h"
#include "RAM.h"
#include "ROM.h"

class Intellivision
{
    friend class EmulationDirector;

    public:
        Intellivision();
        ~Intellivision();
        INT32 turnOn();
        void turnOff();
        void reset();
        void insertCartridge(Cartridge* cartridge);
        Cartridge* getCartridge();
        void removeCartridge();
        void run();
        void setExecImage(INT32* execImage);
        BOOL hasExecImage();
        void setGROMImage(INT32* gromImage);
        BOOL hasGROMImage();
        void setPlugIn(PlugIn* plugIn);
        PlugIn* getPlugIn();
        void enableECSSupport(INT32* image);
        void enableIntellivoiceSupport(INT32* image);
        void setNetworkGame(BOOL b);
        void setPlayersSwitched(BOOL b);
        BOOL arePlayersSwitched();
        void setPlayerOneRemote(BOOL b);
        void setPlayerTwoRemote(BOOL b);

    private:
        void init();
        void addPeripheral(Peripheral* p);
        void removePeripheral(Peripheral* p);

        //audio output lines
        AudioOutputLine*  psgAudioLine;
        AudioOutputLine*  ecsAudioLine;
        AudioOutputLine*  ivoiceAudioLine;

        BOOL              playersSwitched;

        //the current plug-in
        PlugIn*           plugIn;
        BOOL              on;

        //our list of peripherals
        Peripheral*       peripherals[50];
        INT32             periphCount;
    
        //core peripherals
        ProcessorBus      processorBus;
        MemoryBus         memoryBus;
        CP1610            cpu;
        AY38900           stic;
        AY38914           psg;
        AudioMixer        am;
        EmulationDirector emuDirector;
    
        //core memories
        RAM               RAM8bit;
        RAM               RAM16bit;
        ROM               execROM;
        BOOL              hasExec;

        //signal lines
        SignalLine        SR1toINTRMLine;
        SignalLine        SR2toBUSRQLine;
        SignalLine        BUSAKtoSSTLine;

        //hand controllers
        HandController*   player1Controller;
        HandController*   player2Controller;

        //current cartridge in the Intellivision
        Cartridge*        currentCartridge;

        //the ECS peripheral
        ECS             ecs;
        BOOL            ecsInUse;
    
        //the Intellivoice peripheral
        Intellivoice    intellivoice;
        BOOL            intellivoiceInUse;

/*
        void saveGame() throws IOException;
        void loadGame() throws IOException;
        void writeSnapshot() throws IOException;



    protected void fireTurnedOn();
    protected void fireCartridgeChanged(Cartridge oldCart, Cartridge newCart);
    protected void fireErrorOccurred(String errorMessage);
    protected void fireTurnedOff();

*/

/*
    public final static ResourceBundle RESOURCES = ResourceBundle.getBundle(
            "i18n/IntellivisionResources");

    public final static INT32 NTSC_CLOCK_SPEED = 3579545; //in Hz
    public final static INT32 PAL_CLOCK_SPEED  = 4000000; //in Hz


    //output directories
    private File gameSaveDir;
    private File snapshotDir;

    protected BOOL           on;

    //emulation listeners
    protected ArrayList       listeners;

    //existing snapshot file filter
    private FilenameFilter existingSnapFilter = new FilenameFilter() {
            public BOOL accept(File dir, String name) {
                return (name.startsWith(currentCartridge.getType().getName() +
                        "_") && name.endsWith(".png"));
            }
        };
    private FilenameFilter existingSaveFilter = new FilenameFilter() {
            public BOOL accept(File dir, String name) {
                return (name.startsWith(currentCartridge.getType().getName() +
                        "_") && name.endsWith(".sav"));
            }
        };

    //controllers for emulation settings
    private Option[] basicVideoOptions = {
            new ChoiceOption() {
                public String getDescription() {
                    return Intellivision.RESOURCES.getString(
                            "ControllerFrameSkip");
                }
                public Object[] getChoices() {
                    return frameSkipChoices;
                }
                public String getDescription(Object choice) {
                    Object[] args = { choice };
                    return MessageFormat.format(frameSkipFormat, args);
                }
                public void setValue(Object choice) {
                    stic.setFrameSkip(((Integer)choice).INT32Value());
                }
                public Object getValue() {
                    return new Integer(stic.getFrameSkip());
                }
                private String frameSkipFormat = Intellivision.
                        RESOURCES.getString("ControllerFrameSkipFormat");
            }
        };
    private Option[] basicAudioOptions = {
            new ChoiceOption() {
                public String getDescription() {
                    return RESOURCES.getString("ControllerAudioAccuracy");
                }
                public String getDescription(Object choice) {
                    return RESOURCES.getString("ControllerAudioAccuracyOption" +
                            ((Integer)choice).toString());
                }
                public Object[] getChoices() {
                    return choices;
                }
                public Object getValue() {
                    return new Integer(psg.getClockDivisor());
                }
                public void setValue(Object value) {
                    INT32 intValue = ((Integer)value).intValue();
                    psg.setClockDivisor(intValue);
                    ecs.setPSGClockDivisor(intValue);
                }
                private Integer[] choices = {
                        new Integer(1),
                        new Integer(2),
                        new Integer(4),
                        new Integer(8)
                    };
            }
        };

    private Option[] basicInputOptions = new Option[0];
    private Option[] combinedVideoOptions;
    private Option[] combinedAudioOptions;
    private Option[] combinedInputOptions;

    private final static INT32 MAX_PERIPHERALS = 12;

    private final static Integer[] frameSkipChoices = {
            new Integer(0), new Integer(1), new Integer(2),
            new Integer(3), new Integer(4), new Integer(5) };

    class EmulationThread extends Thread
    {
        public void run() {
            try {
                plugIn.init(Intellivision.this);
                fireTurnedOn();
                processorBus.run();
                plugIn.release(Intellivision.this);
                fireTurnedOff();
            }
            catch (EmulationException e) {
                plugIn.release(Intellivision.this);
                fireTurnedOff();
                fireErrorOccurred(e.getPresentableMessage());
            }
            catch (Exception e) {
                plugIn.release(Intellivision.this);
                fireTurnedOff();
                fireErrorOccurred(RESOURCES.getString("ErrorUnexpected"));
            }
        }
    }
*/

};

#endif
