
#include <iostream>
#include "Cartridge.h"
#include "CartridgeType.h"
#include "CartridgeTypeFactory.h"
#include "CRC32.h"
#include "ROM.h"

using namespace std;

Cartridge::Cartridge(const CHAR* name, Memory** memories, INT32 memCount) {
    this->name = name;
    this->memories = memories;
    this->memCount = memCount;
    this->ecs = FALSE;
    this->ivoice = FALSE;
}

Cartridge::Cartridge(const CHAR* name, Memory** memories, INT32 memCount,
    BOOL requiresECS, BOOL usesIntellivoice)
{
    this->name = name;
    this->memories = memories;
    this->memCount = memCount;
    this->ecs = requiresECS;
    this->ivoice = usesIntellivoice;
}

Cartridge::~Cartridge() {
    delete[] memories;
}

const CHAR* Cartridge::getName() {
    return name;
}

INT32 Cartridge::getProcessorCount() {
    return 0;
}

void Cartridge::getProcessor(INT32 i, Processor** p) {
    (*p) = NULL;
}

INT32 Cartridge::getMemoryCount() {
    return memCount;
}

void Cartridge::getMemory(INT32 i, Memory** m) {
    (*m) = memories[i];
}

BOOL Cartridge::requiresECS() {
    return ecs;
}

BOOL Cartridge::usesIntellivoice() {
    return ivoice;
}

Cartridge* Cartridge::loadCartridge(const CHAR* filename) {
    INT32* image;
    INT32  size;
    ROM::loadROMImage(&image, &size, filename, TRUE);

    UINT32 crc = CRC32::getCrc(filename);
    CartridgeType* type = CartridgeTypeFactory::FACTORY.getCartridgeType(crc);

    Cartridge* cart;
    ROM* rom;
    if (type == NULL) {
        Memory** memories = new Memory*[1];
        rom = new ROM((size>>1), 0x5000);
        rom->loadImage(image);
        delete[] image;
        memories[0] = rom;
        cart = new Cartridge("Unknown", memories, 1);
    }
    else if (type->getMemorySpanCount() == 1) {
        Memory** memories = new Memory*[1];
        rom = new ROM(size, type->getMemorySpan(0)->memStart);
        rom->loadImage(image);
        delete[] image;
        memories[0] = rom;
        cart = new Cartridge(type->getName(), memories, 1, type->requiresECS(),
                type->usesIntellivoice());
    }
    else {
        //split the image into multiple smaller memory images
        INT32 memoryCount = type->getMemorySpanCount();
        Memory** memories = new Memory*[memoryCount];
        INT32 nextMemStart = 0;
        for (INT32 i = 0; i < memoryCount; i++) {
            MemorySpan* nextSpan = type->getMemorySpan(i);
            INT32 nextSize = (nextSpan->memEnd - nextSpan->memStart)+1;
            rom = new ROM(nextSize, nextSpan->memStart);
            rom->loadImage(image+nextMemStart);
            memories[i] = rom;
            nextMemStart += nextSize;
        }
        delete[] image;
        cart = new Cartridge(type->getName(), memories, memoryCount,
                type->requiresECS(), type->usesIntellivoice());
    }
    
    return cart;
}

